import math
import numpy as np
import matplotlib.pyplot as plt

def alpha_0(x,wc):
    if wc*wc-x*x > 0:
        return np.sqrt(wc*wc-x*x)
    else:
        return 0
        
#Inputs:
#M: Number of points in y-direction
#N: Number of points in x-direction
#theta_0: Angle of incidence in radians
#a_r: Correlation length divided by lambda
#delta_r: Rms-roughness divided by lambda
#corr_func_choice: 'exp', 'gauss' or 'gauss_aniso'
#num_terms: Number of terms in the sum. High rms-rougness->many terms
#phi_0: azimuthal angle of incidence (Anisotropic corr.func. only)
#a_r_y: Corr.length / lambda in y-direction. (Anisotropic corr.func. only)
#
#Returns:
#qx - x component of lateral scattered momentum vector (array length N)
#qy - x component of lateral scattered momentum vector (array length M)
#mdrc - The incoherent part of the mdrc. (Grid MxN)
def mdrc_kirch(M,N,theta_0,a_r,delta_r,corr_func_choice,num_terms,phi_0=0.0,a_r_y=0.0):
    lamb = 1E-6       #Dummy wavelength.
    wc=2*math.pi/lamb #Length of momentum vectors: (w/c)
    
    a=a_r*lamb         
    delt = delta_r*lamb 
    ay=a_r_y*lamb
    
    #Scattered lateral scattered momentum vector
    qx_arr=np.linspace(-wc,wc,N)
    qy_arr=np.linspace(-wc,wc,M)
    
    #Incident lateral momentum vector and length
    kv=wc*math.sin(theta_0)*np.array([math.cos(phi_0),math.sin(phi_0)])
    klen=wc*np.sin(theta_0)
    
    #Meshgrid of lateral scattered momentum vectors and length
    qx,qy = np.meshgrid(qx_arr, qy_arr, sparse=False, indexing='xy')
    qlen = np.sqrt(qx*qx+qy*qy)
    
    #Make the if-else statement in alpha_0() work for arrays
    valpha_0 = np.vectorize(alpha_0)
    
    #Useful factor
    alph2 = np.power(delt*(valpha_0(qlen,wc) + alpha_0(klen,wc)),2)
    
    #Q_vec=q_vec-k_vec
    Qx=qx-kv[0]
    Qy=qy-kv[1]
    Qx2=Qx*Qx
    Qy2=Qy*Qy
    Q2=Qx2+Qy2
    
    #Do the sum
    mdrc = np.zeros((M,N))
    fac = np.ones((M,N))
    
    #Case gaussian correlation function
    if corr_func_choice == 'gauss':
    	for i in range(1,num_terms):
    		  fac *= (alph2/i) 
    		  mdrc += fac*a*a*math.pi*np.exp(-0.25*Q2*a*a/i)/i
        
    #Case gaussian anisotropic corr.func   
    elif corr_func_choice== 'gauss_aniso':
    	for i in range(1,num_terms):
    		fac *= (alph2/i)   
    		mdrc += fac*a*ay*math.pi*np.exp(-0.25*(Qx2*a*a+Qy2*ay*ay)/i)/i 
    
    #Case exponential corr. func
    elif corr_func_choice == 'exp':
    	for i in range(1,num_terms):
    		fac *= (alph2/i)  
    		mdrc += fac*2*math.pi*a*a*i*np.power(float(i*i)+Q2*a*a,-1.5)          
    else:
        print 'Error. Choose valid correlation function: exp, gauss or gauss_aniso'
        return -1
        
    #Multiply with prefactor
    mdrc *= np.exp(-alph2)/(4*math.pi*math.pi*np.cos(theta_0))
    mdrc *= np.power((wc*wc+valpha_0(qlen,wc)*alpha_0(klen,wc)-qx*kv[0]-qy*kv[1]),2)
    mdrc /= alph2/(delt*delt)

    return qx_arr/wc,qy_arr/wc,mdrc

#Set parameters
M=201
N=401
theta_0=20.0*math.pi/180
phi_0 = 0.0*math.pi/180
delta_r = 0.3
a_r = 2.0
order = 200
corr_func = 'gauss'
a_r_y = 2.0

#Get the mdrc.
qx,qy,mdrc = mdrc_kirch(M,N,theta_0,a_r,delta_r,corr_func,order,0.0,0.0)

#Plot result
cs = plt.contourf(qx,qy,mdrc,500)
plt.colorbar(cs)
